/**
* CSS themes for simplePagination.js
* Author: Flavius Matis - http://flaviusmatis.github.com/
* URL: https://github.com/flaviusmatis/simplePagination.js
*/

ul.simple-pagination {
  list-style: none;
}

.simple-pagination {
  display: block;
  overflow: hidden;
  padding: 0 5px 5px 0;
  margin: 0;
}

.simple-pagination ul {
  list-style: none;
  padding: 0;
  margin: 0;
}

.simple-pagination li {
  list-style: none;
  padding: 0;
  margin: 0;
  float: left;
}

span.ellipse.clickable {
  cursor: pointer;
}

.ellipse input {
  width: 3em;
}

/* ------------------------------------*\
	Compact Theme Styles
\*------------------------------------ */
.compact-theme span {
  cursor: pointer;
}

.compact-theme a,
.compact-theme span {
  float: left;
  color: #333;
  font-size: 14px;
  line-height: 24px;
  font-weight: normal;
  text-align: center;
  border: 1px solid #aaa;
  border-left: none;
  min-width: 14px;
  padding: 0 7px;
  box-shadow: 2px 2px 2px rgba(0, 0, 0, 0.2);
  background: #efefef;

  /* Old browsers */
  background: -moz-linear-gradient(top, #fff 0%, #efefef 100%);

  /* FF3.6+ */
  background: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #fff), color-stop(100%, #efefef));

  /* Chrome,Safari4+ */
  background: -webkit-linear-gradient(top, #fff 0%, #efefef 100%);

  /* Chrome10+,Safari5.1+ */
  background: -o-linear-gradient(top, #fff 0%, #efefef 100%);

  /* Opera11.10+ */
  background: -ms-linear-gradient(top, #fff 0%, #efefef 100%);

  /* IE10+ */
  background: linear-gradient(top, #fff 0%, #efefef 100%);

  /* W3C */
}

.compact-theme a:hover,
.compact-theme li:not(.disabled):not(.active) span:hover {
  text-decoration: none;
  background: #efefef;

  /* Old browsers */
  background: -moz-linear-gradient(top, #efefef 0%, #bbb 100%);

  /* FF3.6+ */
  background: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #efefef), color-stop(100%, #bbb));

  /* Chrome,Safari4+ */
  background: -webkit-linear-gradient(top, #efefef 0%, #bbb 100%);

  /* Chrome10+,Safari5.1+ */
  background: -o-linear-gradient(top, #efefef 0%, #bbb 100%);

  /* Opera11.10+ */
  background: -ms-linear-gradient(top, #efefef 0%, #bbb 100%);

  /* IE10+ */
  background: linear-gradient(top, #efefef 0%, #bbb 100%);

  /* W3C */
}

.compact-theme li:first-child a,
.compact-theme li:first-child span {
  border-left: 1px solid #aaa;
  border-radius: 3px 0 0 3px;
}

.compact-theme li:last-child a,
.compact-theme li:last-child span {
  border-radius: 0 3px 3px 0;
}

.compact-theme .current {
  background: #bbb;

  /* Old browsers */
  background: -moz-linear-gradient(top, #bbb 0%, #efefef 100%);

  /* FF3.6+ */
  background: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #bbb), color-stop(100%, #efefef));

  /* Chrome,Safari4+ */
  background: -webkit-linear-gradient(top, #bbb 0%, #efefef 100%);

  /* Chrome10+,Safari5.1+ */
  background: -o-linear-gradient(top, #bbb 0%, #efefef 100%);

  /* Opera11.10+ */
  background: -ms-linear-gradient(top, #bbb 0%, #efefef 100%);

  /* IE10+ */
  background: linear-gradient(top, #bbb 0%, #efefef 100%);

  /* W3C */
  cursor: default;
}

.compact-theme .ellipse {
  background: #eaeaea;
  padding: 0 10px;
  cursor: default;
}

/* ------------------------------------*\
	Light Theme Styles
\*------------------------------------ */
.light-theme span {
  cursor: pointer;
}

.light-theme a,
.light-theme span {
  float: left;
  color: #666;
  font-size: 14px;
  line-height: 24px;
  font-weight: normal;
  text-align: center;
  border: 1px solid #bbb;
  min-width: 14px;
  padding: 0 7px;
  margin: 0 5px 0 0;
  border-radius: 3px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);
  background: #efefef;

  /* Old browsers */
  background: -moz-linear-gradient(top, #fff 0%, #efefef 100%);

  /* FF3.6+ */
  background: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #fff), color-stop(100%, #efefef));

  /* Chrome,Safari4+ */
  background: -webkit-linear-gradient(top, #fff 0%, #efefef 100%);

  /* Chrome10+,Safari5.1+ */
  background: -o-linear-gradient(top, #fff 0%, #efefef 100%);

  /* Opera11.10+ */
  background: -ms-linear-gradient(top, #fff 0%, #efefef 100%);

  /* IE10+ */
  background: linear-gradient(top, #fff 0%, #efefef 100%);

  /* W3C */
}

.light-theme a:hover,
.light-theme li:not(.disabled):not(.active) span:hover {
  text-decoration: none;
  background: #fcfcfc;
}

.light-theme .current {
  background: #666;
  color: #fff;
  border-color: #444;
  box-shadow: 0 1px 0 rgba(255, 255, 255, 1), 0 0 2px rgba(0, 0, 0, 0.3) inset;
  cursor: default;
}

.light-theme .ellipse {
  background: none;
  border: none;
  border-radius: 0;
  box-shadow: none;
  font-weight: bold;
  cursor: default;
}

/* ------------------------------------*\
	Dark Theme Styles
\*------------------------------------ */
.dark-theme span {
  cursor: pointer;
}

.dark-theme a,
.dark-theme span {
  float: left;
  color: #ccc;
  font-size: 14px;
  line-height: 24px;
  font-weight: normal;
  text-align: center;
  border: 1px solid #222;
  min-width: 14px;
  padding: 0 7px;
  margin: 0 5px 0 0;
  border-radius: 3px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);
  background: #555;

  /* Old browsers */
  background: -moz-linear-gradient(top, #555 0%, #333 100%);

  /* FF3.6+ */
  background: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #555), color-stop(100%, #333));

  /* Chrome,Safari4+ */
  background: -webkit-linear-gradient(top, #555 0%, #333 100%);

  /* Chrome10+,Safari5.1+ */
  background: -o-linear-gradient(top, #555 0%, #333 100%);

  /* Opera11.10+ */
  background: -ms-linear-gradient(top, #555 0%, #333 100%);

  /* IE10+ */
  background: linear-gradient(top, #555 0%, #333 100%);

  /* W3C */
}

.dark-theme a:hover,
.dark-theme li:not(.disabled):not(.active) span:hover {
  text-decoration: none;
  background: #444;
}

.dark-theme .current {
  background: #222;
  color: #fff;
  border-color: #000;
  box-shadow: 0 1px 0 rgba(255, 255, 255, 0.2), 0 0 1px 1px rgba(0, 0, 0, 0.1) inset;
  cursor: default;
}

.dark-theme .ellipse {
  background: none;
  border: none;
  border-radius: 0;
  box-shadow: none;
  font-weight: bold;
  cursor: default;
}
